@@ -1,25 +1,26 @@
 package analyze
 
 import (
-	"errors"
 	"os"
 	"path/filepath"
 )
 
 // File struct
 type File struct {
-	Name      string
-	BasePath  string
-	Size      int64
-	Usage     int64
-	ItemCount int
-	IsDir     bool
-	Files     Files
-	Parent    *File
+	Name           string
+	BasePath       string
+	Flag           rune
+	Size           int64
+	Usage          int64
+	ItemCount      int
+	IsDir          bool
+	Files          Files
+	Parent         *File
+	MutliLinkInode uint64 // Inode number of file with multiple links (hard link)
 }
 
-// ErrNotFound is returned when File item is not found in Files
-var ErrNotFound = errors.New("File item not found")
+// AlreadyCountedHardlinks holds all files with hardlinks that have already been counted
+type AlreadyCountedHardlinks map[uint64]bool
 
 // Path retruns absolute path of the file
 func (f *File) Path() string {
@@ -53,7 +54,7 @@ func (f *File) RemoveFile(file *File) error {
 }
 
 // UpdateStats recursively updates size and item count
-func (f *File) UpdateStats() {
+func (f *File) UpdateStats(links AlreadyCountedHardlinks) {
 	if !f.IsDir {
 		return
 	}
@@ -61,10 +62,22 @@ func (f *File) UpdateStats() {
 	totalUsage := int64(4096)
 	var itemCount int
 	for _, entry := range f.Files {
-		entry.UpdateStats()
+		if entry.IsDir {
+			entry.UpdateStats(links)
+		}
+
+		itemCount += entry.ItemCount
+
+		if entry.MutliLinkInode > 0 {
+			if !links[entry.MutliLinkInode] {
+				links[entry.MutliLinkInode] = true
+			} else {
+				entry.Flag = 'H'
+				continue
+			}
+		}
 		totalSize += entry.Size
 		totalUsage += entry.Usage
-		itemCount += entry.ItemCount
 	}
 	f.ItemCount = itemCount + 1
 	f.Size = totalSize
@@ -74,30 +87,30 @@ func (f *File) UpdateStats() {
 // Files - slice of pointers to File
 type Files []*File
 
-// IndexOf searches File in Files and returns its index, or -1
-func (f Files) IndexOf(file *File) (int, error) {
+// IndexOf searches File in Files and returns its index
+func (f Files) IndexOf(file *File) (int, bool) {
 	for i, item := range f {
 		if item == file {
-			return i, nil
+			return i, true
 		}
 	}
-	return 0, ErrNotFound
+	return 0, false
 }
 
-// FindByName searches name in Files and returns its index, or -1
-func (f Files) FindByName(name string) (int, error) {
+// FindByName searches name in Files and returns its index
+func (f Files) FindByName(name string) (int, bool) {
 	for i, item := range f {
 		if item.Name == name {
-			return i, nil
+			return i, true
 		}
 	}
-	return 0, ErrNotFound
+	return 0, false
 }
 
 // Remove removes File from Files
 func (f Files) Remove(file *File) Files {
-	index, err := f.IndexOf(file)
-	if err != nil {
+	index, ok := f.IndexOf(file)
+	if !ok {
 		return f
 	}
 	return append(f[:index], f[index+1:]...)
@@ -105,8 +118,8 @@ func (f Files) Remove(file *File) Files {
 
 // RemoveByName removes File from Files
 func (f Files) RemoveByName(name string) Files {
-	index, err := f.FindByName(name)
-	if err != nil {
+	index, ok := f.FindByName(name)
+	if !ok {
 		return f
 	}
 	return append(f[:index], f[index+1:]...)
