@@ -6,6 +6,7 @@ import (
 
 	"github.com/dundee/gdu/analyze"
 	"github.com/dundee/gdu/device"
+	"github.com/dundee/gdu/internal/testanalyze"
 	"github.com/dundee/gdu/internal/testdev"
 	"github.com/dundee/gdu/internal/testdir"
 	"github.com/stretchr/testify/assert"
@@ -21,6 +22,7 @@ func TestAnalyzePath(t *testing.T) {
 	ui := CreateStdoutUI(output, false, false, false)
 	ui.SetIgnoreDirPaths([]string{"/xxx"})
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
+	ui.StartUILoop()
 
 	assert.Contains(t, output.String(), "nested")
 }
@@ -39,6 +41,18 @@ func TestAnalyzePathWithColors(t *testing.T) {
 	assert.Contains(t, output.String(), "subnested")
 }
 
+func TestItemRows(t *testing.T) {
+	output := bytes.NewBuffer(make([]byte, 10))
+
+	ui := CreateStdoutUI(output, false, true, false)
+	ui.AnalyzePath("test_dir", testanalyze.MockedProcessDir, nil)
+
+	assert.Contains(t, output.String(), "TiB")
+	assert.Contains(t, output.String(), "GiB")
+	assert.Contains(t, output.String(), "MiB")
+	assert.Contains(t, output.String(), "KiB")
+}
+
 func TestAnalyzePathWithProgress(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
@@ -72,6 +86,21 @@ func TestShowDevicesWithColor(t *testing.T) {
 	assert.Contains(t, output.String(), "xxx")
 }
 
+func TestShowDevicesWithErr(t *testing.T) {
+	output := bytes.NewBuffer(make([]byte, 10))
+
+	getter := device.LinuxDevicesInfoGetter{MountsPath: "/xyzxyz"}
+	ui := CreateStdoutUI(output, false, true, false)
+	err := ui.ListDevices(getter)
+
+	assert.Contains(t, err.Error(), "no such file")
+}
+
+func TestMaxInt(t *testing.T) {
+	assert.Equal(t, 5, maxInt(2, 5))
+	assert.Equal(t, 4, maxInt(4, 2))
+}
+
 func printBuffer(buff *bytes.Buffer) {
 	for i, x := range buff.String() {
 		println(i, string(x))
