@@ -5,11 +5,11 @@ import (
 	"math"
 	"path/filepath"
 	"sort"
-	"strings"
 	"sync"
 	"time"
 
 	"github.com/dundee/gdu/analyze"
+	"github.com/dundee/gdu/common"
 	"github.com/dundee/gdu/device"
 	"github.com/gdamore/tcell/v2"
 	"github.com/rivo/tview"
@@ -38,16 +38,9 @@ const helpText = `
 			  [::b]c    [white:black:-]Sort by items (asc/desc)
 `
 
-// CommonUI is common interface for both terminal UI and text output
-type CommonUI interface {
-	ListDevices(getter device.DevicesInfoGetter)
-	AnalyzePath(path string, analyzer analyze.Analyzer, parentDir *analyze.File)
-	SetIgnoreDirPaths(paths []string)
-}
-
 // UI struct
 type UI struct {
-	app              *tview.Application
+	app              common.Application
 	header           *tview.TextView
 	footer           *tview.TextView
 	currentDirLabel  *tview.TextView
@@ -70,7 +63,7 @@ type UI struct {
 }
 
 // CreateUI creates the whole UI app
-func CreateUI(screen tcell.Screen, useColors bool, showApparentSize bool) *UI {
+func CreateUI(app common.Application, useColors bool, showApparentSize bool) *UI {
 	ui := &UI{
 		askBeforeDelete:  true,
 		sortBy:           "size",
@@ -80,37 +73,30 @@ func CreateUI(screen tcell.Screen, useColors bool, showApparentSize bool) *UI {
 		analyzer:         analyze.ProcessDir,
 	}
 
-	ui.app = tview.NewApplication()
-	ui.app.SetScreen(screen)
+	ui.app = app
 	ui.app.SetInputCapture(ui.keyPressed)
 
 	ui.header = tview.NewTextView()
 	ui.header.SetText(" gdu ~ Use arrow keys to navigate, press ? for help ")
 	if ui.useColors {
-		ui.header.SetTextColor(tcell.ColorWhite)
+		ui.header.SetTextColor(tcell.NewRGBColor(0, 0, 0))
 		ui.header.SetBackgroundColor(tcell.NewRGBColor(36, 121, 208))
 	} else {
-		ui.header.SetTextColor(tcell.ColorBlack)
-		ui.header.SetBackgroundColor(tcell.ColorWhite)
+		ui.header.SetTextColor(tcell.NewRGBColor(0, 0, 0))
+		ui.header.SetBackgroundColor(tcell.NewRGBColor(255, 255, 255))
 	}
 
 	ui.currentDirLabel = tview.NewTextView()
 
 	ui.table = tview.NewTable().SetSelectable(true, false)
 
-	if ui.useColors {
-		ui.table.SetSelectedStyle(tcell.Style{}.
-			Foreground(tcell.ColorBlack).
-			Background(tcell.NewRGBColor(52, 152, 219)))
-	}
-
 	ui.footer = tview.NewTextView().SetDynamicColors(true)
 	if ui.useColors {
-		ui.footer.SetTextColor(tcell.ColorWhite)
+		ui.footer.SetTextColor(tcell.NewRGBColor(0, 0, 0))
 		ui.footer.SetBackgroundColor(tcell.NewRGBColor(36, 121, 208))
 	} else {
-		ui.footer.SetTextColor(tcell.ColorBlack)
-		ui.footer.SetBackgroundColor(tcell.ColorWhite)
+		ui.footer.SetTextColor(tcell.NewRGBColor(0, 0, 0))
+		ui.footer.SetBackgroundColor(tcell.NewRGBColor(255, 255, 255))
 	}
 	ui.footer.SetText(" No items to display. ")
 
@@ -129,11 +115,11 @@ func CreateUI(screen tcell.Screen, useColors bool, showApparentSize bool) *UI {
 }
 
 // ListDevices lists mounted devices and shows their disk usage
-func (ui *UI) ListDevices(getter device.DevicesInfoGetter) {
+func (ui *UI) ListDevices(getter device.DevicesInfoGetter) error {
 	var err error
 	ui.devices, err = getter.GetDevicesInfo()
 	if err != nil {
-		panic(err)
+		return err
 	}
 
 	ui.table.SetCell(0, 0, tview.NewTableCell("Device name").SetSelectable(false))
@@ -164,6 +150,8 @@ func (ui *UI) ListDevices(getter device.DevicesInfoGetter) {
 	ui.table.Select(1, 0)
 	ui.footer.SetText("")
 	ui.table.SetSelectedFunc(ui.deviceItemSelected)
+
+	return nil
 }
 
 // AnalyzePath analyzes recursively disk usage in given path
@@ -201,7 +189,9 @@ func (ui *UI) AnalyzePath(path string, analyzer analyze.Analyzer, parentDir *ana
 			ui.currentDir.Parent = parentDir
 			parentDir.Files = parentDir.Files.RemoveByName(ui.currentDir.Name)
 			parentDir.Files = append(parentDir.Files, ui.currentDir)
-			ui.topDir.UpdateStats()
+
+			links := make(analyze.AlreadyCountedHardlinks, 10)
+			ui.topDir.UpdateStats(links)
 		} else {
 			ui.topDirPath = abspath
 			ui.topDir = ui.currentDir
@@ -215,10 +205,11 @@ func (ui *UI) AnalyzePath(path string, analyzer analyze.Analyzer, parentDir *ana
 }
 
 // StartUILoop starts tview application
-func (ui *UI) StartUILoop() {
+func (ui *UI) StartUILoop() error {
 	if err := ui.app.Run(); err != nil {
-		panic(err)
+		return err
 	}
+	return nil
 }
 
 // SetIgnoreDirPaths sets paths to ignore
@@ -264,8 +255,8 @@ func (ui *UI) showDir() {
 
 	var footerNumberColor, footerTextColor string
 	if ui.useColors {
-		footerNumberColor = "[#e74c3c:#2479d0:b]"
-		footerTextColor = "[white:#2479d0:-]"
+		footerNumberColor = "[#e67100:#2479d0:b]"
+		footerTextColor = "[black:#2479d0:-]"
 	} else {
 		footerNumberColor = "[black:white:b]"
 		footerTextColor = "[black:white:-]"
@@ -332,10 +323,9 @@ func (ui *UI) fileItemSelected(row, column int) {
 func (ui *UI) deviceItemSelected(row, column int) {
 	selectedDevice := ui.table.GetCell(row, column).GetReference().(*device.Device)
 
-	for _, device := range ui.devices {
-		if device.Name != selectedDevice.Name && !strings.HasPrefix(selectedDevice.MountPoint, device.MountPoint) {
-			ui.ignoreDirPaths[device.MountPoint] = true
-		}
+	paths := device.GetNestedMountpointsPaths(selectedDevice.MountPoint, ui.devices)
+	for _, path := range paths {
+		ui.ignoreDirPaths[path] = true
 	}
 
 	ui.AnalyzePath(selectedDevice.MountPoint, ui.analyzer, nil)
@@ -365,11 +355,28 @@ func (ui *UI) confirmDeletion() {
 	ui.pages.AddPage("confirm", modal, true, true)
 }
 
+func (ui *UI) showErr(msg string, err error) {
+	modal := tview.NewModal().
+		SetText(msg + ": " + err.Error()).
+		AddButtons([]string{"ok"}).
+		SetDoneFunc(func(buttonIndex int, buttonLabel string) {
+			ui.pages.RemovePage("error")
+		})
+
+	if !ui.useColors {
+		modal.SetBackgroundColor(tcell.ColorGray)
+	}
+
+	ui.pages.AddPage("error", modal, true, true)
+}
+
 func (ui *UI) deleteSelected() {
 	row, column := ui.table.GetSelection()
 	selectedFile := ui.table.GetCell(row, column).GetReference().(*analyze.File)
 	if err := ui.currentDir.RemoveFile(selectedFile); err != nil {
-		panic(err)
+		msg := "Can't delete " + selectedFile.Name
+		ui.showErr(msg, err)
+		return
 	}
 	ui.showDir()
 	ui.table.Select(min(row, ui.table.GetRowCount()-1), 0)
@@ -545,12 +552,12 @@ func (ui *UI) formatFileRow(item *analyze.File) string {
 		part = int(float64(item.Usage) / float64(item.Parent.Usage) * 10.0)
 	}
 
-	var row string
+	row := string(item.Flag)
 
 	if ui.useColors {
-		row = "[#edb20a:black:b]"
+		row += "[#e67100:black:b]"
 	} else {
-		row = "[white:black:b]"
+		row += "[white:black:b]"
 	}
 
 	if ui.showApparentSize {
@@ -584,7 +591,7 @@ func (ui *UI) formatSize(size int64, reverseColor bool) string {
 	var color string
 	if reverseColor {
 		if ui.useColors {
-			color = "[white:#2479d0:-]"
+			color = "[black:#2479d0:-]"
 		} else {
 			color = "[black:white:-]"
 		}
