@@ -1,6 +1,7 @@
 package tui
 
 import (
+	"os"
 	"path/filepath"
 	"runtime"
 	"sync"
@@ -9,6 +10,8 @@ import (
 
 	"github.com/dundee/gdu/analyze"
 	"github.com/dundee/gdu/device"
+	"github.com/dundee/gdu/internal/testanalyze"
+	"github.com/dundee/gdu/internal/testapp"
 	"github.com/dundee/gdu/internal/testdev"
 	"github.com/dundee/gdu/internal/testdir"
 	"github.com/gdamore/tcell/v2"
@@ -16,12 +19,10 @@ import (
 )
 
 func TestFooter(t *testing.T) {
-	simScreen := tcell.NewSimulationScreen("UTF-8")
+	app, simScreen := testapp.CreateTestAppWithSimScreen(15, 15)
 	defer simScreen.Fini()
-	simScreen.Init()
-	simScreen.SetSize(15, 15)
 
-	ui := CreateUI(simScreen, false, true)
+	ui := CreateUI(app, false, true)
 
 	dir := analyze.File{
 		Name:      "xxx",
@@ -59,26 +60,22 @@ func TestFooter(t *testing.T) {
 }
 
 func TestUpdateProgress(t *testing.T) {
-	simScreen := tcell.NewSimulationScreen("UTF-8")
+	app, simScreen := testapp.CreateTestAppWithSimScreen(15, 15)
 	defer simScreen.Fini()
-	simScreen.Init()
-	simScreen.SetSize(15, 15)
 
 	progress := &analyze.CurrentProgress{Mutex: &sync.Mutex{}, Done: true}
 
-	ui := CreateUI(simScreen, false, false)
+	ui := CreateUI(app, false, false)
 	progress.CurrentItemName = "xxx"
 	ui.updateProgress(progress)
 	assert.True(t, true)
 }
 
 func TestHelp(t *testing.T) {
-	simScreen := tcell.NewSimulationScreen("UTF-8")
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 	defer simScreen.Fini()
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
 
-	ui := CreateUI(simScreen, false, true)
+	ui := CreateUI(app, false, true)
 	ui.showHelp()
 	ui.help.Draw(simScreen)
 	simScreen.Show()
@@ -97,11 +94,9 @@ func TestDeleteDir(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, true, false)
+	ui := CreateUI(app, true, false)
 	ui.askBeforeDelete = false
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
@@ -137,11 +132,9 @@ func TestDoNotDeleteParentDir(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, true, true)
+	ui := CreateUI(app, true, true)
 	ui.askBeforeDelete = false
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
@@ -150,6 +143,7 @@ func TestDoNotDeleteParentDir(t *testing.T) {
 		time.Sleep(100 * time.Millisecond)
 		simScreen.InjectKey(tcell.KeyRune, 'l', 1)
 		time.Sleep(10 * time.Millisecond)
+		// .. is selected now, cannot be deleted
 		simScreen.InjectKey(tcell.KeyRune, 'd', 1)
 		time.Sleep(10 * time.Millisecond)
 		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
@@ -165,11 +159,9 @@ func TestDeleteDirWithConfirm(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, false, false)
+	ui := CreateUI(app, false, false)
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
 
@@ -198,15 +190,52 @@ func TestDeleteDirWithConfirm(t *testing.T) {
 	assert.NoFileExists(t, "test_dir/nested/file2")
 }
 
+func TestDeleteDirWithConfirmNoAskAgain(t *testing.T) {
+	fin := testdir.CreateTestDir()
+	defer fin()
+
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+
+	ui := CreateUI(app, false, false)
+
+	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
+
+	go func() {
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, '?', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyEnter, '1', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'j', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'j', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'd', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRight, ' ', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRight, ' ', 1) // select "do not ask again"
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyEnter, ' ', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
+		time.Sleep(10 * time.Millisecond)
+	}()
+
+	ui.StartUILoop()
+
+	assert.NoFileExists(t, "test_dir/nested/file2")
+}
+
 func TestShowConfirm(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, true, true)
+	ui := CreateUI(app, true, true)
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
 
@@ -241,15 +270,71 @@ func TestShowConfirm(t *testing.T) {
 	assert.FileExists(t, "test_dir/nested/file2")
 }
 
+func TestDeleteWithErr(t *testing.T) {
+	fin := testdir.CreateTestDir()
+	defer fin()
+
+	os.Chmod("test_dir/nested", 0)
+	defer os.Chmod("test_dir/nested", 0755)
+
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+
+	ui := CreateUI(app, true, true)
+	ui.askBeforeDelete = false
+
+	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
+
+	go func() {
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'd', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyEnter, ' ', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
+		time.Sleep(10 * time.Millisecond)
+	}()
+
+	ui.StartUILoop()
+
+	assert.DirExists(t, "test_dir/nested")
+}
+
+func TestDeleteWithErrBW(t *testing.T) {
+	fin := testdir.CreateTestDir()
+	defer fin()
+
+	os.Chmod("test_dir/nested", 0)
+	defer os.Chmod("test_dir/nested", 0755)
+
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+
+	ui := CreateUI(app, false, false)
+	ui.askBeforeDelete = false
+
+	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
+
+	go func() {
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'd', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyEnter, ' ', 1)
+		time.Sleep(10 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
+		time.Sleep(10 * time.Millisecond)
+	}()
+
+	ui.StartUILoop()
+
+	assert.DirExists(t, "test_dir/nested")
+}
+
 func TestRescan(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, true, false)
+	ui := CreateUI(app, true, false)
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
 
@@ -270,17 +355,32 @@ func TestRescan(t *testing.T) {
 	ui.StartUILoop()
 }
 
+// TestItemRows tests that item with different sizes are shown
+func TestItemRows(t *testing.T) {
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+
+	ui := CreateUI(app, true, false)
+
+	ui.AnalyzePath("test_dir", testanalyze.MockedProcessDir, nil)
+
+	go func() {
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 'q', 1)
+		time.Sleep(10 * time.Millisecond)
+	}()
+
+	ui.StartUILoop()
+}
+
 func TestShowDevices(t *testing.T) {
 	if runtime.GOOS != "linux" {
 		return
 	}
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 	defer simScreen.Fini()
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
 
-	ui := CreateUI(simScreen, false, true)
+	ui := CreateUI(app, true, true)
 	ui.ListDevices(getDevicesInfoMock())
 	ui.table.Draw(simScreen)
 	simScreen.Show()
@@ -298,12 +398,10 @@ func TestShowDevicesBW(t *testing.T) {
 		return
 	}
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 	defer simScreen.Fini()
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
 
-	ui := CreateUI(simScreen, true, false)
+	ui := CreateUI(app, false, false)
 	ui.ListDevices(getDevicesInfoMock())
 	ui.table.Draw(simScreen)
 	simScreen.Show()
@@ -316,16 +414,30 @@ func TestShowDevicesBW(t *testing.T) {
 	}
 }
 
+func TestShowDevicesWithError(t *testing.T) {
+	if runtime.GOOS != "linux" {
+		return
+	}
+
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+	defer simScreen.Fini()
+
+	getter := device.LinuxDevicesInfoGetter{MountsPath: "/xyzxyz"}
+
+	ui := CreateUI(app, false, false)
+	err := ui.ListDevices(getter)
+
+	assert.Contains(t, err.Error(), "no such file")
+}
+
 func TestSelectDevice(t *testing.T) {
 	if runtime.GOOS != "linux" {
 		return
 	}
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, true, true)
+	ui := CreateUI(app, true, true)
 	ui.analyzer = analyzeMock
 	ui.SetIgnoreDirPaths([]string{"/proc"})
 	ui.ListDevices(getDevicesInfoMock())
@@ -348,16 +460,18 @@ func TestKeys(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, false, false)
+	ui := CreateUI(app, false, false)
 	ui.askBeforeDelete = false
 
 	ui.AnalyzePath("test_dir", analyze.ProcessDir, nil)
 
 	go func() {
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 's', 1) // sort asc
+		time.Sleep(100 * time.Millisecond)
+		simScreen.InjectKey(tcell.KeyRune, 's', 1) // sort desc
 		time.Sleep(100 * time.Millisecond)
 		simScreen.InjectKey(tcell.KeyRune, 'j', 1)
 		time.Sleep(10 * time.Millisecond)
@@ -388,11 +502,9 @@ func TestSetIgnoreDirPaths(t *testing.T) {
 	fin := testdir.CreateTestDir()
 	defer fin()
 
-	simScreen := tcell.NewSimulationScreen("UTF-8")
-	simScreen.Init()
-	simScreen.SetSize(50, 50)
+	app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
 
-	ui := CreateUI(simScreen, false, true)
+	ui := CreateUI(app, false, true)
 
 	path, _ := filepath.Abs("test_dir/nested/subnested")
 	ui.SetIgnoreDirPaths([]string{path})
@@ -410,7 +522,25 @@ func TestSetIgnoreDirPaths(t *testing.T) {
 	dir := ui.currentDir
 
 	assert.Equal(t, 3, dir.ItemCount)
+}
 
+func TestAppRunWithErr(t *testing.T) {
+	fin := testdir.CreateTestDir()
+	defer fin()
+
+	// app, simScreen := testapp.CreateTestAppWithSimScreen(50, 50)
+	app := testapp.CreateMockedApp(true)
+
+	ui := CreateUI(app, false, true)
+
+	err := ui.StartUILoop()
+
+	assert.Equal(t, "Fail", err.Error())
+}
+
+func TestMin(t *testing.T) {
+	assert.Equal(t, 2, min(2, 5))
+	assert.Equal(t, 3, min(4, 3))
 }
 
 func printScreen(simScreen tcell.SimulationScreen) {
@@ -432,10 +562,14 @@ func getDevicesInfoMock() device.DevicesInfoGetter {
 	item := &device.Device{
 		Name:       "/dev/root",
 		MountPoint: "/",
+		Size:       1e9,
+		Free:       1e3,
 	}
 	item2 := &device.Device{
 		Name:       "/dev/boot",
 		MountPoint: "/boot",
+		Size:       1e12,
+		Free:       1e6,
 	}
 
 	mock := testdev.DevicesInfoGetterMock{}
